<?php

namespace InSegment\WorkersFramework\Services;

use React\Promise;

use InSegment\WorkersFramework\Traits\EventsTrait;
use InSegment\WorkersFramework\Interfaces\QueueInterface;
use InSegment\WorkersFramework\Interfaces\ConsumerInterface;

abstract class AbstractConsumer implements ConsumerInterface
{
    use EventsTrait;
    
    /**
     * Queue instance
     *
     * @var \InSegment\WorkersFramework\Interfaces\QueueInterface
     */
    protected $queue;
    
    /**
     * Whether the consumer is started
     *
     * @var bool
     */
    protected $active = false;
    
    /**
     * Tag to track active subscription
     *
     * @var string|null
     */
    protected $consumerTag;
    
    /**
     * Constructor
     * 
     * @param \InSegment\WorkersFramework\Services\QueueInterface $queue
     */
    public function __construct(QueueInterface $queue)
    {
        $this->queue = $queue;
    }
    
    /** 
     * Activate consumption of the main queue
     * 
     * @return \React\Promise\PromiseInterface
     */
    public function activate(): Promise\PromiseInterface
    {
        if ($this->active) { //already started
            return Promise\resolve();
        }

        if ($this->trigger('starting', $this->queue) !== false) {
            return $this->queue->subscribe($this)->then(function ($consumerTag) {
                $this->active = true;
                $this->consumerTag = $consumerTag;
                $this->trigger('started', $this->queue, $consumerTag);
            });
        }
        
        return Promise\reject();
    }
    
    /**
     * Deactivate consumption of the main queue
     * 
     * @return \React\Promise\PromiseInterface
     */
    public function deactivate(): Promise\PromiseInterface
    {
        if (!$this->active) { // hasn't started
            return Promise\resolve();
        }
        
        $consumerTag = $this->consumerTag;
        if ($this->trigger('stopping', $this->queue, $consumerTag) !== false) {
            return $this->queue->unsubscribe($consumerTag)->then(function () use ($consumerTag) {
                $this->trigger('stopped', $this->queue, $consumerTag);
                $this->active = false;
            });
        }
        
        return Promise\reject();
    }
    
}
